@@ -1,14 +1,14 @@
-;;; brltty.el --- Interface to BrlTTY
+;;; brltty.el --- Interface to BRLTTY
 
-;; Copyright (C) 2004, 2005, 2006 Brailcom, o.p.s.
+;; Copyright (C) 2004, 2005, 2006, 2007 Brailcom, o.p.s.
 
 ;; Author: Milan Zamazal <pdm@brailcom.org>
 
 ;; COPYRIGHT NOTICE
 ;;
-;; This program is free software; you can redistribute it and/or modify
+;; This program is free software: you can redistribute it and/or modify
 ;; it under the terms of the GNU General Public License as published by
-;; the Free Software Foundation; either version 2 of the License, or
+;; the Free Software Foundation, either version 3 of the License, or
 ;; (at your option) any later version.
 ;;
 ;; This program is distributed in the hope that it will be useful,
@@ -17,8 +17,7 @@
 ;; GNU General Public License for more details.
 ;;
 ;; You should have received a copy of the GNU General Public License
-;; along with this program; if not, write to the Free Software
-;; Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+;; along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 ;;; Commentary:
 
@@ -34,16 +33,16 @@
 
 
 (defgroup brltty ()
-  "BrlTTY interface."
+  "BRLTTY interface."
   :group 'speechd-el)
   
 (defcustom brltty-default-host "localhost"
-  "Default BrlTTY host to connect to."
+  "Default BRLTTY host to connect to."
   :type 'string
   :group 'brltty)
 
 (defcustom brltty-default-port '(4101 4102 4103 4104 35751)
-  "Default BrlTTY port to connect to.
+  "Default BRLTTY port to connect to.
 If it is a list, the given port numbers are attempted in the order they are
 given until Emacs connects to something."
   :type 'integer
@@ -55,7 +54,7 @@ given until Emacs connects to something."
   :group 'brltty)
 
 (defcustom brltty-coding 'iso-8859-1
-  "Coding in which texts should be sent to BrlTTY."
+  "Coding in which texts should be sent to BRLTTY."
   :type 'coding-system
   :group 'brltty)
 
@@ -74,7 +73,7 @@ available, from the  environment variable CONTROLVT."
   :group 'brltty)
 
 (defcustom brltty-timeout 3
-  "Maximum number of seconds to wait for a BrlTTY answer."
+  "Maximum number of seconds to wait for a BRLTTY answer."
   :type 'integer
   :group 'brltty)
 
@@ -87,11 +86,33 @@ available, from the  environment variable CONTROLVT."
 
 (defconst brltty--emacs-process-ok (fboundp 'process-put))
 
-(defconst brltty--supported-protocol-versions '(7))
+(defconst brltty--supported-protocol-versions '(8 7))
+
+(defconst brltty--protocol-version-error 13)
+
+(defconst brltty--errors
+  '((1 . "Not enough memory")
+    (2 . "A connection is already running in this tty")
+    (3 . "A connection is already using RAW or suspend mode")
+    (4 . "Not implemented in protocol")
+    (5 . "Forbiden in current mode")
+    (6 . "Out of range or have no sense")
+    (7 . "Invalid size")
+    (8 . "Connection refused")
+    (9 . "Operation not supported")
+    (10 . " Getaddrinfo error")
+    (11 . " Libc error")
+    (12 . " Couldn't find out the tty number")
+    (13 . " Bad protocol version")
+    (14 . " Unexpected end of file")
+    (15 . " Key file empty")
+    (16 . " Packet returned by driver too large")))
 
 (defconst brltty--packet-types
   '(;; commands
-    (authkey . ?K)
+    (version . ?v)
+    (auth . ?a)
+    (authkey . ?K) ; protocol version < 8
     (gettty . ?t)
     (leavetty . ?L)
     (getdisplaysize . ?s)
@@ -101,6 +122,11 @@ available, from the  environment variable CONTROLVT."
     (err . ?e)
     (key . ?k)))
 
+(defconst brltty--authentication-codes
+  '((?N . none)
+    (?K . key)
+    (?C . credentials)))
+
 (defun brltty--add-answer (connection answer)
   (setf (brltty--connection-answers connection)
         (append (brltty--connection-answers connection) (list answer)))
@@ -201,7 +227,7 @@ available, from the  environment variable CONTROLVT."
     (when process
       (delete-process process)
       (setf (brltty--connection-process connection) nil)))
-  (error "Error in communication with BrlTTY: %s" error))
+  (error "Error in communication with BRLTTY: %s" error))
 
 (defun brltty--process-filter* (process output)
   (let ((connection (brltty--process-connection process)))
@@ -260,7 +286,11 @@ available, from the  environment variable CONTROLVT."
   (destructuring-bind (type . data) (brltty--read-packet connection)
     (case type
      (err
-      (error (format "BrlTTY error %d: %s" (brltty--read-integer data) data)))
+      (let ((err-number (brltty--read-integer data)))
+        (if (= err-number brltty--protocol-version-error)
+            (brltty--add-answer connection (list type err-number))
+          (error (format "BRLTTY error %d: %s"
+                         err-number (cdr (assoc err-number brltty--errors)))))))
      (key
       (let ((handler (brltty--connection-key-handler connection)))
         (when handler
@@ -269,6 +299,30 @@ available, from the  environment variable CONTROLVT."
                                 #'brltty--read-integer64
                               #'brltty--read-integer)
                             data)))))
+     (authkey
+      (let ((version (brltty--read-integer (substring data 0 4)))
+            (auth-methods '())
+            (len (length data))
+            (n 4))
+        (while (< n len)
+          (push (cdr (assoc (brltty--read-integer (substring data n (+ n 4)))
+                            brltty--authentication-codes))
+                auth-methods)
+          (setq n (+ n 4)))
+        (brltty--add-answer connection (list type version auth-methods))))
+     (auth
+      (let ((auth-methods '())
+            (len (length data))
+            (n 0))
+        (while (< n len)
+          (push (cdr (assoc (brltty--read-integer (substring data n (+ n 4)))
+                            brltty--authentication-codes))
+                auth-methods)
+          (setq n (+ n 4)))
+        (brltty--add-answer connection (cons type auth-methods))))
+     (version
+      (let ((version (brltty--read-integer (substring data 0 4))))
+        (brltty--add-answer connection (list type version))))
      (nil
       ;; unknown packet type -- ignore
       )
@@ -280,7 +334,7 @@ available, from the  environment variable CONTROLVT."
      (t
       (brltty--add-answer connection (list type data))))))
 
-(defun brltty--read-answer (connection packet-id)
+(defun brltty--read-answer (connection packet-id &optional none-ok)
   (let ((process (brltty--connection-process connection))
         (answer '(nothing-yet)))
     (while (and answer (not (eq (car answer) packet-id)))
@@ -289,8 +343,8 @@ available, from the  environment variable CONTROLVT."
       (unless answer
         (brltty--accept-process-output process)
         (setq answer (brltty--next-answer connection))))
-    (unless answer
-      (error "BrlTTY answer not received"))
+    (when (and (not answer) (not none-ok))
+      (error "BRLTTY answer not received"))
     (cdr answer)))
 
 (defun brltty--send-packet (connection answer packet-id &rest data-list)
@@ -354,37 +408,54 @@ available, from the  environment variable CONTROLVT."
 (put 'brltty-connection-error 'error-conditions
      '(error speechd-connection-error brltty-connection-error))
 (put 'brltty-connection-error 'error-message
-     "Error on opening BrlTTY connection")
+     "Error on opening BRLTTY connection")
 
 (defun brltty-open (&optional host port key-handler)
-  "Open and return connection to a BrlTTY server running on HOST and PORT.
+  "Open and return connection to a BRLTTY server running on HOST and PORT.
 If HOST or PORT is nil, `brltty-default-host' or `brltty-default-port' is used
 respectively."
   (condition-case err
-      (let ((connection nil)
-            (protocol-versions brltty--supported-protocol-versions))
-        (while (and (not connection)
-                    protocol-versions)
-          (let ((connection* (brltty--open-connection host port key-handler)))
-            (condition-case err*
-                (let ((version (car protocol-versions)))
-                  (brltty--send-packet connection* 'ack 'authkey version
-                                       (brltty--authentication-key))
-                  (setf (brltty--connection-protocol-version connection*)
-                        version)
-                  (setq connection connection*))
-              (error
-               (setq protocol-versions (cdr protocol-versions))
-               (unless protocol-versions
-                 (signal (car err*) (cdr err*)))))))
+      ;; In protocol >= 8 server initiates communication, let's look if
+      ;; there is any
+      (let* ((connection (brltty--open-connection host port key-handler))
+             (version (or (first (brltty--read-answer connection 'version t))
+                          7)))
+        (if (> version 7)
+            (progn
+              (unless (member version brltty--supported-protocol-versions)
+                (setq version (apply max brltty--supported-protocol-versions)))
+              (setq auth-methods (brltty--send-packet connection 'auth
+                                                      'version version))
+              (cond
+               ((memq 'none auth-methods)
+                (brltty--send-packet
+                 connection 'ack 'auth
+                 (car (rassoc 'none brltty--authentication-codes))))
+               ((memq 'key auth-methods)
+                (brltty--send-packet
+                 connection 'ack 'auth
+                 (car (rassoc 'key brltty--authentication-codes))
+                 (brltty--authentication-key)))
+               (t
+                (signal 'brltty-connection-error
+                        "No supported BrlAPI authentication method"))))
+          ;; No server initial message, assume protocol version 7 (older
+          ;; versions are not supported)
+          (let ((answer (brltty--send-packet
+                         connection 'ack 'authkey version
+                         (brltty--authentication-key))))
+            (when (equal answer brltty--protocol-version-error)
+              (signal 'brltty-error answer))))
+        (setf (brltty--connection-protocol-version connection) version)
         connection)
     (error
      (signal 'brltty-connection-error err))))
 
 (defun brltty-close (connection)
-  "Close BrlTTY CONNECTION."
+  "Close BRLTTY CONNECTION."
   (when connection
-    (brltty--send-packet connection 'ack 'leavetty)
+    (when (brltty--connection-terminal-spec connection)
+      (brltty--send-packet connection 'ack 'leavetty))
     (let ((process (brltty--connection-process connection)))
       (unless brltty--emacs-process-ok
         (setq brltty--process-connections
@@ -397,12 +468,15 @@ respectively."
   "Return the size of the display as the list (WIDTH HEIGHT)."
   (when connection
     (or (brltty--connection-display-width connection)
-        (setf (brltty--connection-display-width connection)
-              (brltty--send-packet connection 'getdisplaysize
-                                   'getdisplaysize)))))
+        (let ((size (brltty--send-packet connection 'getdisplaysize
+                                         'getdisplaysize)))
+          ;; BrlAPI may report zero size when the display is off.
+          ;; We shouldn't return nor cache such a value.
+          (when (> (car size) 0)
+            (setf (brltty--connection-display-width connection) size))))))
 
 (defun brltty-write (connection text &optional cursor)
-  "Display TEXT in BrlTTY accessed through CONNECTION.
+  "Display TEXT in BRLTTY accessed through CONNECTION.
 TEXT is encoded in the coding given by `brltty-coding' before it is sent.
 CURSOR, if non-nil, is a position of the cursor on the display, starting
 from 0."
@@ -419,17 +493,23 @@ from 0."
                                  (format "%%-%ds" (- display-width
                                                      (length text)))
                                  ""))))
+               (protocol-version (brltty--connection-protocol-version connection))
+               (text-coding brltty-coding)
                (encoded-text (with-speechd-coding-protection
-                              (encode-coding-string text* brltty-coding))))
-          (brltty--send-packet connection nil 'write
-                               38
-                               1 display-width
-                               (length encoded-text) encoded-text
-                               ;; Cursor position may not be too high,
-                               ;; otherwise BrlTTY breaks the connection
-                               (if cursor
-                                   (1+ (min cursor display-width))
-                                 0)))))))
+                              (encode-coding-string text* text-coding)))
+               (arguments (list connection nil 'write
+                                (if (< protocol-version 8) 38 102)
+                                1 display-width
+                                (length encoded-text) encoded-text
+                                ;; Cursor position may not be too high,
+                                ;; otherwise BRLTTY breaks the connection
+                                (if cursor
+                                    (1+ (min cursor display-width))
+                                  0))))
+          (when (>= protocol-version 8)
+            (let ((charset (symbol-name text-coding)))
+              (setf arguments (append arguments (list (vector (length charset)) (upcase charset))))))
+          (apply #'brltty--send-packet arguments))))))
 
 
 ;;; Announce
