@@ -1,4 +1,4 @@
-;;; speechd-brltty.el --- BrlTTY output driver
+;;; speechd-brltty.el --- BRLTTY output driver
 
 ;; Copyright (C) 2004, 2005, 2006 Brailcom, o.p.s.
 
@@ -6,9 +6,9 @@
 
 ;; COPYRIGHT NOTICE
 ;;
-;; This program is free software; you can redistribute it and/or modify
+;; This program is free software: you can redistribute it and/or modify
 ;; it under the terms of the GNU General Public License as published by
-;; the Free Software Foundation; either version 2 of the License, or
+;; the Free Software Foundation, either version 3 of the License, or
 ;; (at your option) any later version.
 ;;
 ;; This program is distributed in the hope that it will be useful,
@@ -17,8 +17,7 @@
 ;; GNU General Public License for more details.
 ;;
 ;; You should have received a copy of the GNU General Public License
-;; along with this program; if not, write to the Free Software
-;; Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+;; along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 ;;; Commentary:
 
@@ -206,6 +205,14 @@ is not recommended to assign or call user commands here."
   "Discard all messages from the display queue."
   (speechd.cancel driver all))
 
+(defun speechd-brltty-command-key (driver key)
+  "Put given key to the command queue."
+  (setq unread-command-events (append unread-command-events (list key))))
+
+(defmacro speechd-brltty-command-key-function (key)
+  "Insert BRLTTY function handling general character KEY event."
+  `(lambda (driver) (speechd-brltty-command-key driver ,key)))
+
 
 ;;; Driver definition, methods and registration
 
