@@ -1,14 +1,14 @@
 ;;; speechd-speak.el --- simple speechd-el based Emacs client
 
-;; Copyright (C) 2003, 2004, 2005, 2006 Brailcom, o.p.s.
+;; Copyright (C) 2003, 2004, 2005, 2006, 2007 Brailcom, o.p.s.
 
 ;; Author: Milan Zamazal <pdm@brailcom.org>
 
 ;; COPYRIGHT NOTICE
 ;;
-;; This program is free software; you can redistribute it and/or modify
+;; This program is free software: you can redistribute it and/or modify
 ;; it under the terms of the GNU General Public License as published by
-;; the Free Software Foundation; either version 2 of the License, or
+;; the Free Software Foundation, either version 3 of the License, or
 ;; (at your option) any later version.
 ;;
 ;; This program is distributed in the hope that it will be useful,
@@ -17,8 +17,7 @@
 ;; GNU General Public License for more details.
 ;;
 ;; You should have received a copy of the GNU General Public License
-;; along with this program; if not, write to the Free Software
-;; Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+;; along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 ;;; Commentary:
 
@@ -228,6 +227,14 @@ Otherwise speak from the point to the end of line on movement by default."
   :type 'boolean
   :group 'speechd-speak)
 
+(defcustom speechd-speak-separator-regexp nil
+  "If non-nil don't read parts of a line separated by the given regexp.
+This is typically useful in w3m where columns are separated only
+by whitespace.  \"   \" may be good value of this variable in such
+a case."
+  :type 'regexp
+  :group 'speechd-speak)
+
 (defcustom speechd-speak-message-time-interval 30
   "Minimum time in seconds, after which the same message may be repeated.
 If the message is the same as the last one, it is not spoken unless the number
@@ -565,9 +572,18 @@ If the prefix argument is given, output the line only from the current point
 to the end of the line."
   (interactive "P")
   (speechd-speak--interactive
-   (speechd-speak-read-region (if rest-only (point) (line-beginning-position))
-                              (line-end-position)
-                              (when (speechd-speak--in-minibuffer-p) ""))))
+   (let ((beg (if rest-only (point) (line-beginning-position)))
+         (end (line-end-position)))
+     (when speechd-speak-separator-regexp
+       (save-match-data
+         (save-excursion
+           (when (re-search-backward speechd-speak-separator-regexp beg t)
+             (setq beg (match-end 0))))
+         (save-excursion
+           (when (re-search-forward speechd-speak-separator-regexp end t)
+             (setq end (match-beginning 0))))))
+     (speechd-speak-read-region beg end
+                                (when (speechd-speak--in-minibuffer-p) "")))))
 
 (defun speechd-speak-read-next-line ()
   "Speak the next line after the current line.
@@ -877,17 +893,19 @@ Language must be an RFC 1766 language code, as a string."
             text))))
 
 (speechd-speak--defadvice (delete-backward-char backward-delete-char) around
-  (speechd-speak--with-command-start-info
-    (let ((speechd-speak--deleted-chars
-           (when speechd-speak-deleted-char
-             (speechd-speak--buffer-substring
-              (point)
-              (max (- (point) (or (ad-get-arg 0) 1))
-                   (point-min))))))
-      ad-do-it
-      (speechd-speak--store-deleted-chars (if speechd-speak-deleted-char
-                                              speechd-speak--deleted-chars
-                                            (format "%c" (preceding-char)))))))
+  (if (speechd-speak--command-start-info)
+      (speechd-speak--with-command-start-info
+        (let ((speechd-speak--deleted-chars
+               (when speechd-speak-deleted-char
+                 (speechd-speak--buffer-substring
+                  (max (- (point) (or (ad-get-arg 0) 1))
+                       (point-min))
+                  (point)))))
+          ad-do-it
+          (speechd-speak--store-deleted-chars (if speechd-speak-deleted-char
+                                                  speechd-speak--deleted-chars
+                                             (format "%c" (preceding-char))))))
+    ad-do-it))
   
 (speechd-speak--defadvice (delete-char) around
   (let ((speechd-speak--deleted-chars
@@ -1962,6 +1980,7 @@ The following symbols are valid state change identifiers: `buffer-name',
 (define-key speechd-speak-mode-map "dp" 'speechd-set-pitch)
 (define-key speechd-speak-mode-map "dr" 'speechd-set-rate)
 (define-key speechd-speak-mode-map "dv" 'speechd-set-voice)
+(define-key speechd-speak-mode-map "d\C-v" 'speechd-set-synthesizer-voice)
 (define-key speechd-speak-mode-map "dV" 'speechd-set-volume)
 
 (defvar speechd-speak--mode-map (make-sparse-keymap))
