@@ -1,14 +1,14 @@
 ;;; speechd.el --- Library for accessing Speech Dispatcher
 
-;; Copyright (C) 2003, 2004, 2005, 2006 Brailcom, o.p.s.
+;; Copyright (C) 2003, 2004, 2005, 2006, 2007 Brailcom, o.p.s.
 
 ;; Author: Milan Zamazal <pdm@brailcom.org>
 
 ;; COPYRIGHT NOTICE
 ;;
-;; This program is free software; you can redistribute it and/or modify
+;; This program is free software: you can redistribute it and/or modify
 ;; it under the terms of the GNU General Public License as published by
-;; the Free Software Foundation; either version 2 of the License, or
+;; the Free Software Foundation, either version 3 of the License, or
 ;; (at your option) any later version.
 ;;
 ;; This program is distributed in the hope that it will be useful,
@@ -17,8 +17,7 @@
 ;; GNU General Public License for more details.
 ;;
 ;; You should have received a copy of the GNU General Public License
-;; along with this program; if not, write to the Free Software
-;; Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+;; along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 ;;; Commentary:
 
@@ -261,6 +260,7 @@ current voice."
     (pause-context . "PAUSE_CONTEXT")
     (capital-character-mode . "CAP_LET_RECOGN")
     (voice . "VOICE")
+    (synthesizer-voice . "SYNTHESIS_VOICE")
     (rate . "RATE")
     (pitch . "PITCH")
     (volume . "VOLUME")
@@ -268,8 +268,11 @@ current voice."
     (output-module . "OUTPUT_MODULE")	; TODO: to be removed sometimes
     ))
 
-(defconst speechd--list-parameter-names
-  '((voices . "VOICES")))
+(defconst speechd--list-parameters
+  '((voices
+     "VOICES" identity)
+    (synthesis-voices
+     "SYNTHESIS_VOICES" (lambda (line) (first (split-string line))))))
 
 (defconst speechd--parameter-value-mappings
   '((message-priority
@@ -714,9 +717,9 @@ Return the opened connection on success, nil otherwise."
 
 
 (defun speechd--list (parameter)
-  (second (speechd--send-command
-	   (list "LIST"
-		 (cdr (assoc parameter speechd--list-parameter-names))))))
+  (multiple-value-bind (command processor)
+      (cdr (assoc parameter speechd--list-parameters))
+    (mapcar processor (second (speechd--send-command (list "LIST" command))))))
 
 
 ;;; Parameter setting functions
@@ -842,6 +845,7 @@ If called with a prefix argument, set it for all connections."
 (speechd--generate-set-command volume "Volume" 0)
 (speechd--generate-set-command rate "Rate" 0)
 (speechd--generate-set-command voice "Voice" 'voices)
+(speechd--generate-set-command synthesizer-voice "Synthesizer voice" 'synthesis-voices)
 (speechd--generate-set-command punctuation-mode "Punctuation mode"
                                speechd--punctuation-modes)
 (speechd--generate-set-command pause-context "Pause context" 0)
